#include <iostream>
#include <queue>
#include <stack>

using namespace std;

// function prototype declarations
void mirror(queue<string>& q);
void stutter(queue<int>& q);
void testMirror();
void testStutter();

int main() {
    testStutter();
    testMirror();
    return 0;
}

/*
 * Replaces every element in the given queue with two of that element.
 * For example, stutter on {1,2,3} produces {1,1,2,2,3,3}.
 */
void stutter(queue<int> &q) {
    int size = q.size();
    for (int i = 0; i < size; i++) {
        int front = q.front();
        q.pop();
        q.push(front);
        q.push(front);
    }
}

/*
 * Adds to the given queue its own contents in reverse order.
 * For example, mirror on {a,b,c} produces {a,b,c,c,b,a}.
 */
void mirror(queue<string> &q) {
    stack<string> s;
    int size = q.size();
    for (int i = 0; i < size; i++) {
        string front = q.front();
        q.pop();
        s.push(front);   // s = {a, b, c}
        q.push(front);   // q = {a, b, c}
    }
    while (!s.empty()) {
        q.push(s.top());
        s.pop();
    }
}

/* Test cases for the stutter function. */
void testStutter() {
    queue<int> q;
    q.push(4);
    q.push(1);
    q.push(8);
    q.push(-2);
    q.push(7);
    cout << "stutter:" << endl;
    cout << "q is ";
    int n = q.size();
    for (int i  = 0; i < n; ++i) {
        cout << q.front() << " ";
        int front = q.front();
        q.pop();
        q.push(front);
    }
    cout << endl;
    stutter(q);
    cout << "q is ";
    n = q.size();
    for (int i  = 0; i < n; ++i) {
        cout << q.front() << " ";
        int front = q.front();
        q.pop();
        q.push(front);
    }
    cout << endl;
    cout << endl;
}

/* Test cases for the mirror function. */
void testMirror() {
    queue<string> q;
    q.push("abc");
    q.push("def");
    q.push("ghi");
    q.push("jkl");
    cout << "mirror:" << endl;
    cout << "q is ";
    int n = q.size();
    for (int i  = 0; i < n; ++i) {
        cout << q.front() << " ";
        string front = q.front();
        q.pop();
        q.push(front);
    }
    cout << endl;
    mirror(q);
    cout << "q is ";
    n = q.size();
    for (int i  = 0; i < n; ++i) {
        cout << q.front() << " ";
        string front = q.front();
        q.pop();
        q.push(front);
    }
    cout << endl;
}
