#include <iostream>
#include <iomanip>
#include <stack>

using namespace std;

// function prototype declarations
int checkBalance(string code);
void testBalance(string s, int expected);

int main() {
    // test the checkBalance function
    cout << "        index 0123456789012345678901234567890123456789" << endl;
    testBalance("if (a(4) > 9) { foo(a(2)); }", -1);
    testBalance("for (i=0;i<a(3};i++) { foo{); )", 14);
    testBalance("while (true) foo(); }{ ()", 20);
    testBalance("})})})", 0);
    testBalance("{foo({x}, {{y}}), ()(hi{you});{foo();}}", -1);
    testBalance("if (a(4) > 9) { foo(a(2)); }(", 28);
    return 0;
}

/*
 * Examines all () and {} pairs in the given string of code and 
 * finds mismatches, where a ) doesn't follow a (, or where a
 * } doesn't follow a {, etc.  All opened parentheses and brackets
 * must be closed in the opposite order.
 */
int checkBalance(string code) {
    stack<char> parens;
    for (size_t i = 0; i < code.length(); i++) {
        char c = code[i];
        if (c == '(' || c == '{') {
            parens.push(c);
        } else if (c == ')') {
            if (parens.empty() || parens.top() != '(') {
                return i;
            }
            parens.pop();
        } else if (c == '}') {
            if (parens.empty() || parens.top() != '{') {
                return i;
            }
            parens.pop();
        }
    }
    
    if (!parens.empty()) {
        return code.length() - 1;
    }
    
    return -1;
}

/* Test cases for the checkBalance function. */
void testBalance(string s, int expected) {
    string temp = "checkBalance(\"" + s + "\")";
    cout << left << setw(57) << temp << " => ";
    int actual = checkBalance(s);
    cout << right << setw(2) << actual;
    if (expected != actual) {
        cout << " (FAIL; should be " << setw(2) << expected << ")";
    }
    cout << endl;
}
